/* CaptainDomo - the Java mailing list server
 *
 * CaptainDomo.java
 *
 * Copyright (c) 1997 by Xavier Machenaud. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 * Use at your own risk. I do not guarantee the fitness of
 * CaptainDomo for any purpose, and I do not accept responsibility
 * for any damage you do to yourself or others by using CaptainDomo.
 */

import java.io.*;
import java.util.*;


class ListMember {

	// Member name
	public String name = null;				// Member name
	public boolean forward = false;			// Send mail back to the author

	public ListMember( String s, boolean b)
	{
		name    = s;
		forward = b;
	}
}

class ListServer {

	// Config File
	private String configFile = null;

	// SMTP
	private String SmtpServer = null;

	// POP
	private String PopServer = null;
	private int PopServerPort = 110;
	private String user = null;
	private String password = null;

	// Mailing List
	private String MailingListAddress = null;
	private int CheckInterval = 5;	// By default, check every 5 minutes
	private Vector members = null;
	private boolean OnlyMembersCanPost = true;	// Only members are allowed to mail to the list

	public ListServer()
	{
	}

	public ListServer(String config)
	{
		configFile = config;
		try {
			if (!readConfigFile()) {
				System.exit(0);
			}
		}
		catch (IOException e)
		{
			System.exit(0);
		}
	}

	public int checkInterval()
	{
		return CheckInterval;
	}

	private boolean readConfigFile() 
			throws IOException
	{
		FileInputStream InputFile = new FileInputStream(configFile);

		DataInputStream InputData = new DataInputStream(InputFile);

		String line = null;

		// *** POP3 ***
		do {
			line=InputData.readLine();
		} while (line!= null && line.charAt(0)=='#');
		if (line==null) {
			System.err.println("POP3 server expected");
			return false;
		}

		// pop3 server
		PopServer = line;

		do {
			line=InputData.readLine();
		} while (line!= null && line.charAt(0)=='#');
		if (line==null) {
			System.err.println("POP3 user expected");
			return false;
		}

		// user
		user = line;

		do {
			line=InputData.readLine();
		} while (line!= null && line.charAt(0)=='#');
		if (line==null) {
			System.err.println("POP3 user's password expected");
			return false;
		}

		// password
		password = line;

		// *** SMTP ***
		do {
			line=InputData.readLine();
		} while (line!= null && line.charAt(0)=='#');
		if (line==null) {
			System.err.println("SMTP server expected");
			return false;
		}

		// smtp server
		SmtpServer = line;

		// Mailing List
		do {
			line=InputData.readLine();
		} while (line!= null && line.charAt(0)=='#');
		if (line==null) {
			System.err.println("Mailing list address expected");
			return false;
		}

		// mailing list address
		MailingListAddress = line;

		do {
			line=InputData.readLine();
		} while (line!= null && line.length()>0 && line.charAt(0)=='#');
		//if (line==null) {
		//	System.err.println("Mailing list members expected");
		//	return false;
		//}

		// member list
		// * contain at list one member with optional boolean
		//   saying if mail should be send back to the author.
		// * terminated by an empty line
		members = new Vector(); // empty the list
		if (line!=null && line.length()>0) {
			do {
				String name = null;
				boolean forward = false;
				StringTokenizer lineToken = new StringTokenizer(line);
				name = lineToken.nextToken();

				if (lineToken.hasMoreTokens()) {
					Boolean bool = new Boolean(lineToken.nextToken());
					forward = bool.booleanValue();
				}

				ListMember member = new ListMember(name, forward);
				
				members.addElement(member);

				do {
					line=InputData.readLine();
				} while (line!= null && line.length()>0 && line.charAt(0)=='#');
			} while (line!=null && line.length()>0) ;
		}


		if (line!=null) {
			do {
				line=InputData.readLine();
			} while (line!= null && line.charAt(0)=='#');
			if (line!=null) {
				// check every ? minutes
				// This isn't required
				CheckInterval = Integer.parseInt(line);
			}
		}

		InputFile.close();

		return true;
	}

	private boolean writeConfigFile()
			throws IOException
	{
		FileOutputStream OutputFile = new FileOutputStream(configFile);
		PrintStream OutputData = new PrintStream(OutputFile);

		OutputData.println("# *** POP3 ***");
		OutputData.println("# pop3 server");
		OutputData.println(PopServer);
		OutputData.println("# pop3 user");
		OutputData.println(user);
		OutputData.println("# pop3 user's password");
		OutputData.println(password);
		OutputData.println("# *** SMTP ***");
		OutputData.println("# smtp server");
		OutputData.println(SmtpServer);
		OutputData.println("# *** Mailing List ***");
		OutputData.println("# mailing list address");
		OutputData.println(MailingListAddress);
		OutputData.println("# mailing list members (terminated by an empty line)");
		OutputData.println("# contain the member's name followed by 'true' if the member");
		OutputData.println("# wants to receive the mail he posts to the list.");
		OutputData.println("# Only the name is required");
		for (Enumeration e = members.elements() ; e.hasMoreElements() ;) {
			ListMember member = (ListMember)e.nextElement();
			Boolean bool= new Boolean(member.forward);
			OutputData.println(member.name+" "+bool.toString());
		}
		OutputData.println();
		OutputData.println("# check time interval (in minutes)");
		OutputData.println("# this field is not required");
		OutputData.println(CheckInterval);

		OutputFile.close();

		return true;
	}

	private ListMember getMember(String name)
	{
		if (name==null)
			return null;
                        
		for (Enumeration enum = members.elements() ; enum.hasMoreElements() ;) {
			ListMember guy = (ListMember)enum.nextElement();

			if (guy!=null && name.equalsIgnoreCase(guy.name))
				return guy;
		}

		return null;
	}

	private boolean sendMailToGuy(String guy, String message)
	{
		try {
			MySmtpClient smtp = new MySmtpClient(SmtpServer);
			//System.out.println("Connected to "+SmtpServer);
			smtp.from("<"+MailingListAddress+">");
			smtp.to("<"+guy+">");
			PrintStream smtpMsg = smtp.startMessage();
			smtpMsg.println(message);
			smtp.closeServer();
			//System.out.println("Send ["+message+"] to "+guy);
			return true;
		}
		catch (IOException e) {
			System.err.println("Can't send ["+message+"] to "+guy);
			return false;
		}
	}

	private boolean sendMailToList(String message, String from)
	{
		try {
			MySmtpClient smtp = new MySmtpClient(SmtpServer);
			//System.out.println("Connected to "+SmtpServer);
			smtp.from("<"+MailingListAddress+">");
			// Send mail to all members
			for (Enumeration enum = members.elements() ; enum.hasMoreElements() ;) {
				//String guy = (String)enum.nextElement();
				ListMember guy = (ListMember)enum.nextElement();

				// Don't send mail to sender
				if (from.indexOf(guy.name)==-1 || guy.forward) {
					smtp.to("<"+guy.name+">");
					//System.out.println("Mail sent to "+guy.name);
				}
				else {
					System.out.println("Don't send mail to sender ("+guy.name+")");
				}
			}
			PrintStream smtpMsg = smtp.startMessage();
			smtpMsg.print(message);
			smtp.closeServer();
			System.out.println("Message from ("+from+") posted to <"+MailingListAddress+">");
			//System.out.println("send :\n****begin****\n"+message+"\n****end****\n to the list");
			return true;
		}
		catch (Exception e) {
			System.err.println("Problem sending mail from "+from+".\nError: "+e.toString());
			return false;
		}
	}


	private boolean isValidCommand(String subject)
	{

		return (	subject!=null
				&&	( subject.equalsIgnoreCase("subscribe")
					||	subject.equalsIgnoreCase("unsubscribe")
					||	subject.equalsIgnoreCase("repro")
					||	subject.equalsIgnoreCase("norepro")
					||	subject.equalsIgnoreCase("list")
					||	subject.equalsIgnoreCase("help")
					)
				);
	}


	private void processCommand(String subject, String from)
	{
		ListMember member;

		// Everyone can use the help command
		if (subject.equalsIgnoreCase("help")) {
			StringBuffer buffer = new StringBuffer("");
			buffer.append("The following command are available:\r\n");
			buffer.append("   help        -> help !!!\r\n");
			buffer.append("   list        -> list members\r\n");
			buffer.append("   subscribe   -> subscribe to the mailing list\r\n");
			buffer.append("   unsubscribe -> unsubscribe from the mailing list\r\n");
			buffer.append("   repro       -> send back mail to the author\r\n");
			buffer.append("   norepro     -> don't send back mail to the author (default)\r\n");
			sendMailToGuy(from, buffer.toString());
		}

		// Everyone can use the subscribe command
		else if (subject.equalsIgnoreCase("subscribe")) {
			if ((member = getMember(from))==null) {
				// send welcome mail
				if ( sendMailToGuy( from, 
                                                    "You ("+from+") have been added to the mailing list"))
                                	{
					// Add new member
					member = new ListMember(from, false);
					//member = new ListMember(from, true);
					members.addElement(member);
                                        }
                                else {
					System.out.println("subscription error: Couldn't send email to ["+from+"]\n");
                                }
			}
			else {
				// send error mail
				sendMailToGuy(from, "You ("+from+") are already member of the mailing list");
			}
		}

		// You need to be member of the list to use the other commands
		else if ((member = getMember(from))==null) {
			// send error mail
			sendMailToGuy(from, "You ("+from+") are not member of the mailing list");
		}

		else if (subject.equalsIgnoreCase("unsubscribe")) {
			// Remove member
			members.removeElement(member);

			// send goodbye mail
			sendMailToGuy(from, "You've been removed from the mailing list");
		}

		else if (subject.equalsIgnoreCase("repro")) {
			member.forward = true;
			// send welcome mail
			sendMailToGuy(from, "You ("+from+") will now receive the mail you send to the mailing list");
		}

		else if (subject.equalsIgnoreCase("norepro")) {
			member.forward = false;
			// send welcome mail
			sendMailToGuy(from, "You ("+from+") won't now receive the mail you send to the mailing list");
		}

		else if (subject.equalsIgnoreCase("list")) {
			StringBuffer buffer = new StringBuffer("");
			buffer.append("Here are the members of the list:\r\n");
			for (Enumeration e = members.elements() ; e.hasMoreElements() ;) {
				ListMember guy = (ListMember)e.nextElement();
				buffer.append(guy.name+"\r\n");
			}
			sendMailToGuy(from, buffer.toString());
		}
		else {
			sendMailToGuy(from, "Unknown command ["+subject+"] : send 'help' in the subject to get the list of all commands");
		}

	}


	public void ProcessMail()
	{
		Pop3 pop = new Pop3(PopServer, PopServerPort, user, password);

		System.out.println("Try to connect to "+PopServer+" POP server");

		if (!pop.connect()) {
			System.err.println("Can't connect to "+PopServer+" POP server");
			return;
		}

		System.out.println("Connected to "+PopServer+" POP server");

		int msgNb = pop.getMsgCount();

		System.out.println(msgNb+" message(s) are waiting at "+PopServer);

		for (int i=1; i<=msgNb ; i++) {

			InternetTextMessage msg = pop.getMsg(i);

			if (msg.isNull()) {
				System.out.println("Error while getting message["+i+"]");
				break;
			}
			else {
				System.out.println("Process Message["+i+"]");

				boolean deleteMsg = false;

				//System.out.println(msg.getText());
				//System.out.println();

				String subjectField = msg.getTag("Subject");
				String fromField = InternetTextMessage.GetAddressFromFieldValue(msg.getTag("From"));

				// Remove trailing spaces in the subject
				if ((subjectField != null) && (subjectField.endsWith(" "))) {
					int lastSpace = subjectField.lastIndexOf(" ");
					while (subjectField.charAt(lastSpace-1)==' ') {
						lastSpace = lastSpace - 1;
					}
					String newSubject = subjectField.substring(0, lastSpace);
					subjectField = newSubject;
				}								

				if (isValidCommand(subjectField)) {
					// Mailing List Command
					processCommand(subjectField, fromField);


					// Write the new config file
					try {
					if (!writeConfigFile()) {
						System.err.println("can't write in config file "+configFile);
					}
					else {
						deleteMsg = true;
					}
					}
					catch (Exception e) {
						System.err.println("can't write in config file "+configFile);
					}
				}
				else if (fromField!=null && fromField.length()>0) {
									// Do I have to put < > ???
					msg.addTag("To", "<"+MailingListAddress+">", "From");

					//System.out.println(withReplyTo);
					if (getMember(fromField)==null && OnlyMembersCanPost) {
						System.out.println(fromField+" isn't a member -> don't send mail");
						/***
						// replace listadmin@host.com by the list administrator email address
						deleteMsg = sendMailToGuy("listadmin@host.com", msg.getText() + "\n***** MESSAGE NOT DELIVERED ****\n");
						***/
						StringBuffer buffer = new StringBuffer("");
						buffer.append("Only members are allowed to send mail to "+MailingListAddress+"\r\n");
						buffer.append("\r\n-------------------------- begin message --------------------------\r\n");
						buffer.append(msg.getText());
						buffer.append("\r\n-------------------------- end message ----------------------------\r\n");
						deleteMsg = sendMailToGuy(fromField, buffer.toString());
					}
					else {
						msg.addTag("Reply-To", "<"+MailingListAddress+">", "To");
						deleteMsg = sendMailToList(msg.getText(), fromField);
					}
				}
				
				if (deleteMsg) {
					if (pop.deleteMsg(i))
						System.out.println("Message["+i+"] deleted");
					else
						System.err.println("Can't delete message["+i+"]");
				}
				//System.out.println();
			}
		}

		pop.close();
		System.out.println("Disconnected from "+PopServer+" POP server");

	}
}

class ListServerThread extends Thread
{
	private ListServer listServer;

	public ListServerThread(String ConfigFile)
	{
		listServer = new ListServer(ConfigFile);
		setDaemon(true);
	}

	public void run()
	{
		for (;true;) {
			listServer.ProcessMail();
			try {
				sleep(listServer.checkInterval()*60*1000);
			}
			catch (Exception e) {
				System.out.println("Program aborted");
				break;
			}
		}
	}
}


class CaptainDomo {
	public static void main(String[] args)
	{
		if (args.length==0) {
			ListServerThread mailingList = new ListServerThread("CaptainDomo.ini");
			mailingList.start();
		}
		else {
			for (int i=0; i < args.length; i++) {
				ListServerThread mailingList = new ListServerThread(args[i]);
				mailingList.start();
			}
		}

		for (;true;) {
			Thread.currentThread().suspend();
			System.out.println("Exit from suspend");
		}
	}
}
