/* CaptainDomo - the Java mailing list server
 *
 * InternetTextMessage.java
 *
 * Copyright (c) 1997 by Xavier Machenaud. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies.
 *
 * Use at your own risk. I do not guarantee the fitness of
 * CaptainDomo for any purpose, and I do not accept responsibility
 * for any damage you do to yourself or others by using CaptainDomo.
 */

/**
 * see RFC 822 "Standard for ARPA Internet Text Messages"
 */
class InternetTextMessage {
	private String	Text = null;

	public void setText(String pText) {
		Text = pText;
	}

	public String getText() {
		return Text;
	}

	public boolean isNull() {
		return Text == null;
	}

	private boolean isSpace(int idx) {
		return Text.charAt(idx)=='\t' || Text.charAt(idx)==' ' ;
	}

	private boolean isEndOfLine(int idx) {
		return Text.charAt(idx)=='\n' ;
	}

	private boolean isEndOfField(int idx) {
		return isEndOfLine(idx) && !isSpace(idx+1) ;
	}

	private static String removeComments(String str) {
		int i=0;
		boolean inComment = false;
		boolean quoted = false;
		StringBuffer buffer = new StringBuffer("");
		StringBuffer spaceBuffer = new StringBuffer("");

		// skip leading spaces
		while (i<str.length() && (str.charAt(i)=='\t' || str.charAt(i)==' '))
			i++;

		for (; i<str.length() ; i++) {
			if (!quoted && str.charAt(i) == '(')
				inComment = true;
			else if (!quoted && str.charAt(i) == ')')
				inComment = false;
			else if (quoted || !inComment) {
				if (str.charAt(i) == '"')
					quoted = !quoted;
				if (str.charAt(i)=='\t' || str.charAt(i)==' ')
					spaceBuffer.append(str.charAt(i));
				else {
					if (spaceBuffer.length() > 0) {
						buffer.append(spaceBuffer.toString());
						spaceBuffer = new StringBuffer("");
					}
					buffer.append(str.charAt(i));
				}
			}
		}

		return buffer.toString();
	}

	public static String GetAddressFromFieldValue(String value) {
		// suppose that the from string is :
		// * either "Xavier Machenaud" <xavier.machenaud@usa.net>
		// * either xavier.machenaud@usa.net
		// The string can also contain comment -- between '(' and ')'.
		// This doesn't implement all RFC 822 address but it shoulkd work!!!!

		String address = removeComments(value);
		//System.out.print("Address:["+value+"], Uncommented Address:["+address+"]\n");

		int idx = address.lastIndexOf(' ');
		String name;

		//System.out.print("idx:["+idx+"], Address length:["+address.length()+"]\n");        
                
		if (idx!=-1) {
			// The value is "Xavier Machenaud" <xavier.machenaud@usa.net>
			// Don't forget to remove the '<>'
			name = address.substring(idx+2, address.length()-1);
		}
		else {
			// The value is xavier.machenaud@usa.net or <xavier.machenaud@usa.net>
			if (address.charAt(0)=='<') {
				name = address.substring(1, address.length()-1);
			} else {
				name = address;
			}
		}

		//System.out.print("return Address:["+name+"]\n");
		return name;
	}

	public String addTag(String tag, String value, String beforeTag)
	{
		// Could need to search the tag with the string "\n<tag>:"
		// because I found a example where the message start with
		// From xavier.machenaud@usa.net Thu Aug 21 13:30:36 1997
		// (come from the pop.hotmail.com)

		// Don't handle empty message
		if (Text==null)
			return null;

		String newMsg = null;
		int idx;
		int endHeaderIdx = -1;

		// BE CARREFULL : the header finish after the empty line
		endHeaderIdx = Text.indexOf("\n\n");

		if ((idx = Text.indexOf('\n'+tag))==-1 || idx > endHeaderIdx) {
			// Look for the tag where to add before
			int beforeTagIdx = -1;
			if (beforeTag!=null)
				beforeTagIdx = Text.indexOf('\n'+beforeTag);

			// If the tag doesn't exist, add at the end of the headers
			if (beforeTagIdx==-1)
				beforeTagIdx = endHeaderIdx;

			newMsg = Text.substring(0, beforeTagIdx);
			idx = beforeTagIdx;
			newMsg = newMsg + "\n" + tag + ": " + value + '\n';
			while (!isEndOfField(idx))
				idx++;
			String removedsubstring = Text.substring(beforeTagIdx, idx);
			//System.out.print("removedsubstring : [" + removedsubstring +"]\n");
			String substring = Text.substring(++idx, Text.length());
			//System.out.print("substring : [" + substring +"]\n");
			newMsg = newMsg + substring;
		}
		else {
			// we're keeping the '\n'
			idx++;
			// build a new string
			newMsg = Text.substring(0, idx);
			int savedIdx = idx;
			newMsg = newMsg + tag + ": " + value + '\n';
			while (!isEndOfField(idx))
				idx++;
			String removedsubstring = Text.substring(savedIdx, idx);
			//System.out.print("removedsubstring : [" + removedsubstring +"]\n");
			String substring = Text.substring(++idx, Text.length());
			//System.out.print("substring : [" + substring +"]\n");
			newMsg = newMsg + substring;
			//System.out.print("newMsg : \n" + newMsg +"\n");
		}

		Text = newMsg;
		return Text;
	}

	public String getTag(String tag)
	{
		String newMsg = null;
		int idx;

		if (Text==null)
			return null;
                
		// BE CARREFULL : the header finish after the empty line
		int headerIdx = Text.indexOf("\n\n");
		//System.out.print("end of the header at car " + headerIdx+"\n");
		if (!((idx = Text.indexOf('\n'+tag))==-1 || idx > headerIdx)) {
			// pass the '\n'+tag+':'
			idx = idx + tag.length() + 2;

			// skip spaces
			while (isSpace(idx))
				idx++;

			// get the string
			StringBuffer buffer = new StringBuffer("");
			while (!isEndOfField(idx)) {
				buffer.append(Text.charAt(idx));
				idx++;
			}
			newMsg = buffer.toString();
		}

		return newMsg;
	}
}